;; This file can really only be used if you
;; have preformed both:
;; (asdf:operate 'asdf:load-op :ele-bdb)
;; and 
;; (asdf:operate 'asdf:load-op :ele-clsql)

(in-package :ele-tests)

(deftest remove-element
    (let ((a (vector 'a 'b 'c))
	  (ans (vector 'a 'c)))
      (setf a (ele::remove-indexed-element-and-adjust 1 a))
      (and (equal (aref a 0) (aref ans 0))
	   (equal (aref a 1) (aref ans 1))
	   (equal (length a) (length ans))))
  t)


(deftest migrate1
    (let ((old-store *store-controller*)
	  (*prev-commit* *auto-commit*)
	  (*auto-commit* t)
	  (rv nil))
	  (unwind-protect 
	       (let (
		     (sc1 (open-store-bdb *testdb-path*))
		     (sc2 (open-store-sql *testpg-path*)))
		 (add-to-root "x" "y" :store-controller sc1)
		 (copy-from-key "x" sc1 sc2)
		 (setf rv (equal (get-from-root "x" :store-controller sc1)
				 (get-from-root "x" :store-controller sc2))))
	    (progn
	    (setq *store-controller* old-store)
	    (setq *auto-commit* *prev-commit*)))
      rv)
  t)


(deftest migrate2
    (let ((old-store *store-controller*)
	  (*prev-commit* *auto-commit*)
	  (*auto-commit* t)
	  (rv nil))
      (unwind-protect 
	   (let
	       ((sc1 (open-store-bdb *testdb-path*))
		(sc2 (open-store-sql *testpg-path*)))
	     (let ((ibt (build-btree sc1)))
	       (loop for i from 0 to 10
		  do
		    (setf (get-value i ibt) (* i i)))
	       (let ((mig (migrate sc2 ibt)))
		 (btree-differ ibt mig))))
	    (progn
	      (setq *store-controller* old-store)
	      (setq *auto-commit* *prev-commit*))))
  nil)


(deftest migrate3
    (let ((old-store *store-controller*)
	  (*prev-commit* *auto-commit*)
	  (*auto-commit* t)
	  (rv nil))
      (unwind-protect 
	   (let ((sc1 (open-store-bdb *testdb-path*))
		 (sc2 (open-store-sql *testpg-path*))
		 )
	     (let* ((ibt (build-indexed-btree sc1)))
	       (let (
		     (index
		      (add-index ibt :index-name 'crunch :key-form 'crunch
				 :populate t))
		     )
		 (loop for i from 0 to 10
		    do
		    (setf (get-value i ibt) (* i i)))
		 (let* ((mig (migrate sc2 ibt))
			(nindex (gethash 'crunch (indices ibt))))
		   (loop for i from 0 to 10
		      do
		      (if (not 
			   (equal
			    (get-value i index)
			    (get-value i nindex)			
			    ))
			  (progn
			    (format t "YIKES ~A ~%" i)
			    )))
		   (setf rv (not (btree-differ ibt mig)))
		   ))))
	(progn
	  (setq *store-controller* old-store)
	  (setq *auto-commit* *prev-commit*)))
      rv)
  t)


(deftest migrate4
    (finishes
      (let ((old-store *store-controller*)
	    (*prev-commit* *auto-commit*)
	    (*auto-commit* t)
	    (rv nil))
	(unwind-protect 
	     (let* (
		   (sc1 (open-store-bdb *testdb-path*))
		   (sc2 (open-store-sql *testpg-path*))
		   )
	       (let* ((ibt (build-indexed-btree sc1)))
		 (let (
		       (index
			(add-index ibt :index-name 'crunch :key-form 'crunch
				   :populate t))
		       (x 0)
		       )
		   (loop for i from 0 to 10
		      do
		      (setf (get-value i ibt) (* i i)))
		   )))
	(progn
	  (setq *store-controller* old-store)
	  (setq *auto-commit* *prev-commit*)))
      ))
  t)

(deftest migrate5
    (finishes
      (let ((old-store *store-controller*)
	    (*prev-commit* *auto-commit*)
	    (*auto-commit* t))
	(unwind-protect 
	     (let ((osc (if (subtypep (type-of *store-controller*) 'sql-store-controller)
			   (open-store-bdb *testdb-path*)
			   (open-store-sql *testpg-path*)
			   )))
;; really need to test the an error is thrown when attempting to migrate
;; non-persistent object!
	     (let* ((f1 (make-instance 'pfoo  :sc *store-controller*))
		    (f2 (make-instance 'pfoo :slot1 "this is a string"  :sc *store-controller*))
		    (b1 (make-instance 'pbar :slot2 "another string"  :sc *store-controller*))
		    )		 
	       (let ((fm1 
		      (ele::migraten-pobj 
		       osc f1
		       #'(lambda (dst src)
			   (if (slot-boundp src 'slot1)
			   (setf (slot1 dst) (slot1 src))))))
		     (fm2 
		      (ele::migraten-pobj 
		       osc f2
		       #'(lambda (dst src)
			   (if (slot-boundp src 'slot1)
			   (setf (slot1 dst) (slot1 src))))))
		     (bm1 (ele::migraten-pobj 
			   osc b1
			   #'(lambda (dst src)
			   (if (slot-boundp src 'slot2)
			       (setf (slot2 dst) (slot2 src))))))
			  )
		     (and 
		      (and (not (slot-boundp fm1 'slot1))
			   (not (slot-boundp f1 'slot1)))
		      (equal (slot1 fm2) (slot1 f2))
		      (equal (slot2 bm1) (slot2 b1))))))
	(progn
	  (setq *store-controller* old-store)
	  (setq *auto-commit* *prev-commit*))))
      )
  t)
